local LIBLA = LibStub:GetLibrary("LibLoadedAddons")
local gps = LibStub("LibGPS2")
local CHAT_LIB = LibStub('libChat-1.0')

TweakIt = {
    addonsLoaded = {},
}

------------------------------------------------------------------
-- Initialize Variables --
------------------------------------------------------------------

local ADDON_NAME = "TweakIt"
local CODE_VERSION = 4.8

-- Updated, need to remove code references:
TweakIt.name = ADDON_NAME
TweakIt.codeVersion = CODE_VERSION


--====================================--
--==== DO NOT CHANGE =================--
TweakIt.savedVarVersion = 2.1
--====================================--


------------------------------------------------------------------
-- Initialize Strings  --
------------------------------------------------------------------

local colorDrkOrange = "|cFFA500" -- Dark Orange
local colorYellow = "|cFFFF00" -- yellow

--============================================--
--===== String Inits =====--
--============================================--
--[[
ZO_CreateStringId("TWEAKIT_BANKEDGOLD",  colorYellow..GetString(SI_CURRENCY_YOUR_BANKED_GOLD).. ":|r  <<X:1>> |t16:16:EsoUI/Art/currency/currency_gold.dds|t")
ZO_CreateStringId("TWEAKIT_INVGOLD",  colorYellow..GetString(SI_CURRENCY_YOUR_GOLD).. ":|r  <<X:1>> |t16:16:EsoUI/Art/currency/currency_gold.dds|t")
--]]
--ZO_CreateStringId("TWEAKIT_BANK_SPACE",  zo_format(SI_INVENTORY_BANK_REMAINING_SPACES, 1,2))
------------------------------------------------------------------
-- Default Saved Vars  --
------------------------------------------------------------------
local defaultAccountSavedVars = {
    ["REPAIR_EQUIPPED_KEYBIND_BUTTON"] = false,
    ["HIDE_TARGET_FRAME"] = false,
    ["HIDE_TARGET_FRAME_ALWAYS"] = false,
    ["DEFAULT_TO_COD"] = false,
    ["HERO_DOLL_AT_BANK_MERCHANT"] = false,
    ["AUTO_REMOVE_WAYPOINT"] = false,
    ["AUTO_REMOVE_WAYPOINT_MESSAGES"] = false,
    --=================================================--
    -- Removed, The game now has this feature built in --
    --=================================================--
    ["HIDE_EXTENDED_INV_INFO"] = true,
    ["HIDE_ATTRIBUTE_BARS"] = false,
    ["LOCKPICK_STOP_SIGN"] = false,
    ["ZOOM_MODULE_ENABLED"] = false,
    ["WORLD_MAP_MAX_ZOOM"] = 0,
    ["WORLD_MAP_DEFAULT_ZOOM"] = 5,
    ["WORLD_MAP_ZOOM_FAST_TRAVEL"] = 1,
    ["WORLD_MAP_ZOOM_FAST_TRAVEL_KEEP"] = 1,
    ["WORLD_MAP_ZOOM_ZONE"] = 5,
    ["WORLD_MAP_ZOOM_WORLD"] = 5,
    ["WORLD_MAP_ZOOM_SUBZONE"] = 5,
    ["WORLD_MAP_ZOOM_DUNGEON"] = 5,
    ["WORLD_MAP_ZOOM_AVA"] = 5,
    ["WORLD_MAP_ZOOM_AVA_RESPAWN"] = 1,
    ["RECALL_COOLDOWN_TIMER"] = false,
    ["RECALL_COOLDOWN_WINDOW"] = false, -- make it visible permanently
    ["RECALL_COOLDOWN_OFFSETX"] = 100,
    ["RECALL_COOLDOWN_OFFSETY"] = 100,
    ["CURRENT_WEAPON_WINDOW"] = false,
    ["CURRENT_WEAPON_OFFSETX"] = 100,
    ["CURRENT_WEAPON_OFFSETY"] = 100,
    ["CURRENT_WEAPON_ICON_SIZE"] = 64,
    ["CURRENT_WEAPON_HIDE_TEXT"] = false,
    ["KILL_SOUND"] = SOUNDS.SKILL_XP_DARK_FISSURE_CLOSED,
    ["CURRENT_KILL_WINDOW"] = false,
    ["CURRENT_KILL_WINDOW_OFFSETX"] = 100,
    ["CURRENT_KILL_WINDOW_OFFSETY"] = 100,
    ["IMPROVE_AUTO_MAX_BOOSTERS"] = false,
    ["IMPROVE_100SUCCESS_SKIP_DIALOG"] = false,
    ["CAMPAIGN_QUE_WINDOW"] = false,
    ["CAMPAIGN_QUE_AUTOJOIN"] = false,
    ["CAMPAIGN_QUE_AUTOJOIN_TIME"] = 5,
    ["CAMPAIGN_QUE_OFFSETX"] = 100,
    ["CAMPAIGN_QUE_OFFSETY"] = 200,
    ["WAYPOINT_WINDOW"] = false,
    ["WAYPOINT_DELTA_SCALE"] = 3,
    ["WAYPOINT_WINDOW_OFFSETX"] = 100,
    ["WAYPOINT_WINDOW_OFFSETY"] = 100,
    ["RESIZE_COMPASS"] = false,
    ["RESIZE_COMPASS_SIZE"] = 40,
    ["SELLER_WHISPER_ADD_LINK"] = false,
    ["SELL_JUNK_DIALOG"] = false,
    ["ITEM_LINK_APPEND_TRAIT"] = false,
}
-- PlaySound("Raid_Trial_Failed")
local defaultCharSavedVars = {
    ["CURRENT_WEAPON_WINDOW_CHAR_OVERRIDE"] = false,
}

local tCampaignStates = {
    [CAMPAIGN_QUEUE_REQUEST_STATE_CONFIRMING] = "Confirming",
    [CAMPAIGN_QUEUE_REQUEST_STATE_FINISHED] = "Finished",
    [CAMPAIGN_QUEUE_REQUEST_STATE_PENDING_ACCEPT] = "Pending Acceptance",
    [CAMPAIGN_QUEUE_REQUEST_STATE_PENDING_JOIN] = "Pending Join",
    [CAMPAIGN_QUEUE_REQUEST_STATE_PENDING_LEAVE] = "Pending Leave",
    [CAMPAIGN_QUEUE_REQUEST_STATE_WAITING] = "Waiting in que.",
}

--- Safely checks that specified option enabled.
function TweakIt.isOptionEnabled(optionName)
    return TweakIt.SavedVariables and TweakIt.SavedVariables[optionName]
end

------------------------------------------------------------------
-- Initialize Bindings  --
------------------------------------------------------------------
local function CreateBindings()

    local colorMagenta = "|cFF00FF" -- Magenta
    local colorDrkOrange = "|cFFA500" -- Dark Orange

    ZO_CreateStringId("SI_BINDING_NAME_TWEAKIT_ONECLICK_LOGOUT", colorDrkOrange .. "Logout|r" .. colorMagenta .. " - Allows you to log out in one click.")

    ZO_CreateStringId("SI_BINDING_NAME_TWEAKIT_RELOADUI", colorDrkOrange .. "ReloadUI" .. colorMagenta .. " - Allows you to reload the UI with a hotkey.")
end

-- Called from the event EVENT_MAIL_OPEN_MAILBOX
-- Automatically select the C.O.D. option based on saved var
local function OnMailOpen()
    if TweakIt.SavedVariables["DEFAULT_TO_COD"] then
        ZO_MailSend_SetCoDMode()
    end
end


-- RegisterUpdate function to check current loc vs waypoint loc
-- Used for Automatically removing waypoints
local function CheckWaypointLoc()
    local mapType = GetMapType()
    -- cant get coordinates from the cosmic map
    if mapType == MAPTYPE_COSMIC then return end

    local wpX, wpY = GetMapPlayerWaypoint()
    local pX, pY = GetMapPlayerPosition("player")
    local pingDelta = TweakIt.SavedVariables["WAYPOINT_DELTA_SCALE"]

    local dx = (wpX - pX)
    local dy = (wpY - pY)
    local distSq = dx * dx + dy * dy
    local pingDist = 0

    if GetMapType() == MAPTYPE_WORLD then
        pingDist = 1e-7 * pingDelta
    elseif GetMapType() == MAPTYPE_SUBZONE then
        pingDist = 1e-4 * pingDelta
    else
        pingDist = 5e-6 * pingDelta
    end

    if distSq < pingDist then
        --if zo_floatsAreEqual(iWaypointOffsetX, iPlayerOffsetX, percX) and zo_floatsAreEqual(iWaypointOffsetY, iPlayerOffsetY, percY) then
        if TweakIt.SavedVariables["AUTO_REMOVE_WAYPOINT_MESSAGES"] then
            CENTER_SCREEN_ANNOUNCE:AddMessage(0, CSA_EVENT_SMALL_TEXT, SOUNDS.ACHIEVEMENT_AWARDED, "You have reached your waypoint.")
        end
        EVENT_MANAGER:UnregisterForUpdate(TweakIt.name .. "RemoveWaypoint")
        RemovePlayerWaypoint()
    end
end

-- Initialize waypoints for Automatically removing waypoints
-- Must be done to register a new RegisterForUpdate to persist through reloadUI's
function TweakIt.InitWaypoint()
    if not TweakIt.SavedVariables["AUTO_REMOVE_WAYPOINT"] then return end

    local iWaypointOffsetX, iWaypointOffsetY = GetMapPlayerWaypoint()
    -- if no waypoint then return
    if iWaypointOffsetX == 0 and iWaypointOffsetY == 0 then return end

    EVENT_MANAGER:RegisterForUpdate(TweakIt.name .. "RemoveWaypoint", 300, function() CheckWaypointLoc() end)
end

-- Automatically remove waypoints
local function OnMapPing(eventCode, pingEventType, pingType, pingTag, offsetX, offsetY)
    -- sound == nil, check for libGPS, it nils the ping sound when it sets
    -- waypoints to measure the map. We dont want events firing then.
    --if SOUNDS.MAP_PING == nil then return end
    -- change in libGps code allows us to use this instead now:
    if gps:IsMeasuring() then return end

    if not TweakIt.SavedVariables["AUTO_REMOVE_WAYPOINT"] then return end

    if (pingEventType == PING_EVENT_ADDED) and pingTag == "waypoint" then
        if TweakIt.SavedVariables["AUTO_REMOVE_WAYPOINT_MESSAGES"] then
            CENTER_SCREEN_ANNOUNCE:AddMessage(0, CSA_EVENT_SMALL_TEXT, SOUNDS.ACHIEVEMENT_AWARDED, "Waypoint Set")
        end

        EVENT_MANAGER:RegisterForUpdate(TweakIt.name .. "RemoveWaypoint", 300, function() CheckWaypointLoc() end)
    elseif (pingEventType == PING_EVENT_REMOVED) and pingTag == "waypoint" then
        EVENT_MANAGER:UnregisterForUpdate(TweakIt.name .. "RemoveWaypoint")
    end
end

local function OnCombatEvent(eventCode, result, isError, abilityName, abilityGraphic, abilityActionSlotType, sourceName, sourceType, targetName, targetType, hitValue, powerType, damageType, log)
    if result ~= ACTION_RESULT_DIED and result ~= ACTION_RESULT_DIED_XP then return end

    local sSource = zo_strformat(SI_UNIT_NAME, sourceName)
    if sSource ~= GetUnitName("player") then return end

    local sTarget = zo_strformat(SI_UNIT_NAME, targetName)
    if not sTarget or sTarget == "" then return end

    --local message = GetString("SI_ACTIONRESULT", result)

    if TweakIt.SavedVariables["KILL_SOUND"] ~= "None" then
        PlaySound(TweakIt.SavedVariables["KILL_SOUND"])
    end

    if TweakIt.SavedVariables["CURRENT_KILL_WINDOW"] then
        local sAbility = zo_strformat(SI_ABILITY_NAME, abilityName)
        local sKillMessage = "You Killed " .. sTarget .. " with " .. sAbility
        local iStringWidth = TweakItKillWindowLabel:GetStringWidth(sKillMessage)

        TweakItKillWindowLabel:SetText(sKillMessage)
        TweakItKillWindow:SetDimensions(iStringWidth + 20, 40)
        TweakItKillWindow:SetAlpha(1)
        TweakItKillWindow:SetHidden(false)
        TweakItKillWindow.timeline:PlayFromStart()
    end
    --[[
    if result == ACTION_RESULT_DIED then
    elseif result == ACTION_RESULT_DIED_XP then
        d("ACTION_RESULT_DIED_XP: "..GetString("SI_ACTIONRESULT", result))
    elseif result == ACTION_RESULT_KILLING_BLOW then
        d("ACTION_RESULT_KILLING_BLOW: "..GetString("SI_ACTIONRESULT", result))
    end
    --]]
end

local function InitLockpickStopSign()
    local tlw = CreateTopLevelWindow("LockPickIt_Stop", GuiRoot)
    tlw:SetDimensions(384, 384)
    tlw:SetHidden(true)
    tlw:SetAnchor(CENTER, GuiRoot, CENTER)
    tlw:SetDrawLevel(666) -- Ok, I'm drunk and I want to set it to 666. Don't ask me later why. © Upya
    tlw:SetDrawLayer(DL_OVERLAY)
    tlw:SetDrawTier(DT_HIGH)

    local StopCntrl = WINDOW_MANAGER:CreateControl("LockPickIt_StopTexture", tlw, CT_TEXTURE)
    StopCntrl:SetAnchorFill()
    StopCntrl:SetTexture("TweakIt/textures/LockPick_Stop.dds")

    TweakIt.stopSign = tlw
end

-- Show Current Weapon: Called from event to update current weapon
local function OnActiveWeaponPairChanged(eventCode, activeWeaponPair, locked)
    TweakIt.UpdateCurrentWeapons(activeWeaponPair)
end

-- Show Current Weapon: Used to update current weapon during equip/unequip
-- Can't hook EquipItem or UnequipItem because the slotIndex is nilable for weapons
local function OnSingleSlotUpdate(eventCode, bagId, slotId, isNewItem, itemSoundCategory, updateReason)
    if bagId ~= BAG_WORN then return end
    local equipSlots = {
        [EQUIP_SLOT_MAIN_HAND] = true,
        [EQUIP_SLOT_OFF_HAND] = true,
        [EQUIP_SLOT_BACKUP_MAIN] = true,
        [EQUIP_SLOT_BACKUP_OFF] = true,
    }
    if equipSlots[slotId] then
        TweakIt.UpdateCurrentWeapons()
    end
end

--[[
-- Waypoint Window
local function onZoneChanged()
	TweakIt.UpdateWaypointWindow()
end
--]]

-- Campaign Que
local function OnCampaignQueJoined(eventCode, campaignId, isGroup)
    if not TweakIt.SavedVariables["CAMPAIGN_QUE_WINDOW"] then return end

    TweakIt.UpdateCampaignWindow(campaignId, isGroup, nil, nil)
end

local function OnCampaignQueLeft(eventCode, campaignId, isGroup)
    if not TweakIt.SavedVariables["CAMPAIGN_QUE_WINDOW"] then return end

    TweakItCampaignQue:SetHidden(true)
    EVENT_MANAGER:UnregisterForUpdate(TweakIt.name .. "CampaignTime")
end

local function OnCampaignQuePositionChanged(eventCode, campaignId, isGroup, position)
    if not TweakIt.SavedVariables["CAMPAIGN_QUE_WINDOW"] then return end

    TweakIt.UpdateCampaignWindow(campaignId, isGroup, nil, position)
end

local function OnCampaignQueStateChanged(eventCode, campaignId, isGroup, state)
    if not TweakIt.SavedVariables["CAMPAIGN_QUE_WINDOW"] then return end

    if state == CAMPAIGN_QUEUE_REQUEST_STATE_CONFIRMING then
        TweakItCampaignQue.BtnAccept:SetEnabled(true)

        local function PlayAnnouncement(_bResetSetting)
            CENTER_SCREEN_ANNOUNCE:AddMessage(EVENT_ACHIEVEMENT_AWARDED, CSA_EVENT_LARGE_TEXT, SOUNDS.ACHIEVEMENT_AWARDED, colorDrkOrange .. "Campaign Ready")
        end

        --enable game sounds if game doesn't have focus
        if not (DoesGameHaveFocus() or GetSetting_Bool(SETTING_TYPE_AUDIO, AUDIO_SETTING_BACKGROUND_AUDIO)) then
            --SetSetting(integer system, integer settingId, string value, integer setOptions)
            SetSetting(SETTING_TYPE_AUDIO, AUDIO_SETTING_BACKGROUND_AUDIO, "true")
            PlayAnnouncement()
            EVENT_MANAGER:RegisterForEvent(TweakIt.name, EVENT_GAME_FOCUS_CHANGED, function(evt, hasFocus)
                if hasFocus then
                    EVENT_MANAGER:UnregisterForEvent(TweakIt.name, EVENT_GAME_FOCUS_CHANGED)
                    SetSetting(SETTING_TYPE_AUDIO, AUDIO_SETTING_BACKGROUND_AUDIO, "false")
                end
            end)
        else
            PlayAnnouncement()
        end

    else
        TweakItCampaignQue.BtnAccept:SetEnabled(false)
    end
    if state == CAMPAIGN_QUEUE_REQUEST_STATE_CONFIRMING or state == CAMPAIGN_QUEUE_REQUEST_STATE_WAITING then
        TweakItCampaignQue.BtnLeave:SetEnabled(true)
    else
        TweakItCampaignQue.BtnLeave:SetEnabled(false)
    end

    TweakIt.UpdateCampaignWindow(campaignId, isGroup, state, nil)
end




-- On Event Player Activated
local function OnPlayerActivated(eventCode)
    TweakIt.CheckRecallCooldown()

    if TweakIt.SavedVariables["RESIZE_COMPASS"] then
        local savedHeight = TweakIt.SavedVariables["RESIZE_COMPASS_SIZE"]
        TweakIt.ChangeCompassHeight(savedHeight)
    end

    if not LIBLA:IsAddonLoaded("KeybindSearch") then
        -- Initialize Keybindings Search Box
        TweakIt.SetUpKeybindingsSearch()
    end

    EVENT_MANAGER:UnregisterForEvent(TweakIt.name, EVENT_ADD_ON_LOADED)
end

-----------------------------------------------------------------------------------------
-- Initialize Function --
-----------------------------------------------------------------------------------------
local function Initialize()
    -- Registers addon to loadedAddon library
    LIBLA:RegisterAddon(ADDON_NAME, CODE_VERSION)

    TweakIt.SavedVariables = ZO_SavedVars:NewAccountWide("TweakItSavedVars", TweakIt.savedVarVersion, nil, defaultAccountSavedVars)
    TweakIt.SavedVariablesChar = ZO_SavedVars:New("TweakItSavedVars", TweakIt.savedVarVersion, nil, defaultCharSavedVars)

    TweakIt.CreateSettingsMenu()
    CreateBindings()

    -- Initialize the store keybindstripDescriptors
    TweakIt.InitStoreKeybindStripDescriptors()

    -- Initialize Repair Equipped Confirmation Dialog
    TweakIt.InitRepairEquippedDialog()

    -- Initializes the Hide target frame feature
    TweakIt.InitHideTargetFrame()

    -- Initializes hero doll options to show at bank & merchant
    TweakIt.InitHeroDoll()

    -- Initialize World Map Max Zoom
    TweakIt.InitMaxWorldMapZoom()

    -- Initialize Waypoint for Auto-Remove Waypoints
    TweakIt.InitWaypoint()

    -- Initialize Recall Cooldown
    TweakIt.InitRecallCooldown()

    -- Initialize Current Weapon Window
    TweakIt.InitCurrentWeapon()

    -- Initialize Kill Window
    TweakIt.InitKillWindow()

    -- Initialize Campaign Window
    TweakIt.InitCampaignQueWindow()

    --=================================================--
    -- Removed, The game now has this feature built in --
    --=================================================--
    -- Initialize all ExtendedInfoBar Code
    ExtendedInfoBar_Initialize()

    -- Initialize Lockpicking StopSign
    InitLockpickStopSign()
    -------------------------
    -- Register for Events --
    -------------------------
    EVENT_MANAGER:RegisterForEvent(TweakIt.name, EVENT_INVENTORY_SINGLE_SLOT_UPDATE, OnSingleSlotUpdate)
    EVENT_MANAGER:RegisterForEvent(TweakIt.name, EVENT_MAIL_OPEN_MAILBOX, OnMailOpen)
    EVENT_MANAGER:RegisterForEvent(TweakIt.name, EVENT_MAP_PING, OnMapPing)
    EVENT_MANAGER:RegisterForEvent(TweakIt.name, EVENT_PLAYER_ACTIVATED, OnPlayerActivated)
    EVENT_MANAGER:RegisterForEvent(TweakIt.name, EVENT_ACTIVE_WEAPON_PAIR_CHANGED, OnActiveWeaponPairChanged)

    local eventDiedName = TweakIt.name .. "ResultDied"
    local eventDiedXpName = TweakIt.name .. "ResultDiedXp"
    EVENT_MANAGER:RegisterForEvent(eventDiedName, EVENT_COMBAT_EVENT, OnCombatEvent)
    EVENT_MANAGER:AddFilterForEvent(eventDiedName, EVENT_COMBAT_EVENT, REGISTER_FILTER_COMBAT_RESULT, ACTION_RESULT_DIED)

    EVENT_MANAGER:RegisterForEvent(eventDiedXpName, EVENT_COMBAT_EVENT, OnCombatEvent)
    EVENT_MANAGER:AddFilterForEvent(eventDiedXpName, EVENT_COMBAT_EVENT, REGISTER_FILTER_COMBAT_RESULT, ACTION_RESULT_DIED_XP)

    EVENT_MANAGER:RegisterForEvent(TweakIt.name, EVENT_CAMPAIGN_QUEUE_JOINED, OnCampaignQueJoined)
    EVENT_MANAGER:RegisterForEvent(TweakIt.name, EVENT_CAMPAIGN_QUEUE_LEFT, OnCampaignQueLeft)
    EVENT_MANAGER:RegisterForEvent(TweakIt.name, EVENT_CAMPAIGN_QUEUE_POSITION_CHANGED, OnCampaignQuePositionChanged)
    EVENT_MANAGER:RegisterForEvent(TweakIt.name, EVENT_CAMPAIGN_QUEUE_STATE_CHANGED, OnCampaignQueStateChanged)

    --EVENT_MANAGER:RegisterForEvent(TweakIt.name, EVENT_ZONE_CHANGED, onZoneChanged)
    --EVENT_MANAGER:RegisterForEvent(TweakIt.name, EVENT_OPEN_FENCE, OnFenceOpen)
    --EVENT_MANAGER:RegisterForEvent(TweakIt.name, EVENT_CLOSE_FENCE, OnFenceClose)




    -----------------------
    -- Unregister Events --
    -----------------------
end


--- Functionality to add item trait name after each link in chat.
local function escapeMagic(text)
    return (text:gsub('[%^%$%(%)%%%.%[%]%*%+%-%?]', '%%%1'))
end

--- Extends all links with information about item trait.
local function extendLinksInText(text)
    -- WARNING - No logs here - they will cause infinity loop.
    if not TweakIt.SavedVariables or not TweakIt.SavedVariables["ITEM_LINK_APPEND_TRAIT"] then return text end
    for link in string.gmatch(text, "%|H[%a%d]+:%a+[:%d]+%|h[^%|]-%|h") do
        local linkText, color, linkType = ZO_LinkHandler_ParseLink(link)
        if (linkType == ITEM_LINK_TYPE) then
            local extendedLink
            local traitIndex = GetItemLinkTraitInfo(link)
            if traitIndex ~= ITEM_TRAIT_TYPE_NONE then
                local traitText = GetString("SI_ITEMTRAITTYPE", traitIndex)
                extendedLink = link .. "(" .. traitText .. ")"
                text = text:gsub(escapeMagic(link), extendedLink)
            end
        end
    end
    return text
end

--EVENT_MANAGER:RegisterForUpdate("TweakIt_debug", 2000, function() d("Debug |H1:item:68495:363:50:0:0:0:0:0:0:0:0:0:0:0:0:37:0:1:0:7950:0|h|h") end)
CHAT_LIB:registerText(function(channelID, from, text, isCustomerService) return extendLinksInText(text) end, "TweakIt_appendTraitToLinks")
--[[
local OnChatMessageReceived = ZO_ChatSystem_GetEventHandlers()[EVENT_CHAT_MESSAGE_CHANNEL]
if OnChatMessageReceived then
    local function hookOnChatMessage(messageType, fromName, text, ...)
        text = extendLinksInText(text)
        return OnChatMessageReceived(messageType, fromName, text, ...)
    end

    ZO_ChatSystem_AddEventHandler(EVENT_CHAT_MESSAGE_CHANNEL, hookOnChatMessage)
end

]]

local oldD
local function hookChatForLinkEntending()
    oldD = CHAT_SYSTEM.AddMessage
    function CHAT_SYSTEM:AddMessage(text)
        if text then text = extendLinksInText(text) end
        return oldD(self, text)
    end
end
hookChatForLinkEntending()

--- Adds hook to extend native context menu of trade house slot.
local function configureTradeHouseContextHook()
    --actually, copypaste with inventoryslot.lua
    local function GetTradingHouseSearchResultLink(inventorySlot, linkStyle)
        return GetTradingHouseSearchResultItemLink(ZO_Inventory_GetSlotIndex(inventorySlot), linkStyle)
    end

    local function GetTradingHouseListingLink(inventorySlot, linkStyle)
        return GetTradingHouseListingItemLink(ZO_Inventory_GetSlotIndex(inventorySlot), linkStyle)
    end

    local function whisperToSeller(inventorySlot)
        local sellerName = inventorySlot.sellerName
        if MasterMerchant then
            --d("!" .. sellerName .. " ! " .. sellerName:len() .. "!" .. string.find(sellerName, "@"))
            -- All the best to author of MM...
            sellerName = zo_strgsub(zo_strsplit(';', sellerName), "^%s*(.-)\|c000000%s*$", "%1")
            --d("!" .. sellerName .. "!" .. sellerName:len())
        end
        StartChatInput("", CHAT_CHANNEL_WHISPER, sellerName)
        if not TweakIt.SavedVariables["SELLER_WHISPER_ADD_LINK"] then
            return
        end
        local type = ZO_InventorySlot_GetType(inventorySlot)
        local linkFunction
        local typeSwitch = {
            [SLOT_TYPE_TRADING_HOUSE_ITEM_RESULT] = GetTradingHouseSearchResultLink,
            [SLOT_TYPE_TRADING_HOUSE_ITEM_LISTING] = GetTradingHouseListingLink,
        }
        local link = ZO_LinkHandler_CreateChatLink(typeSwitch[type], inventorySlot)
        --GetTradingHouseListingItemLink(ZO_Inventory_GetSlotIndex(inventorySlot), LINK_STYLE_BRACKETS)
        --       or GetTradingHouseSearchResultItemLink(ZO_Inventory_GetSlotIndex(inventorySlot), LINK_STYLE_BRACKETS)
        ZO_LinkHandler_InsertLink(zo_strformat(SI_TOOLTIP_ITEM_NAME, link))
    end

    --- Callback for item selection. Checks that item has seller and allows to whisper.
    -- @param inventorySlot selected control data.
    -- @param slotActions already configured actions.
    local function onHookNeContextMenuItem(inventorySlot, slotActions)
        local type = ZO_InventorySlot_GetType(inventorySlot)
        --d("Slot type: " .. type .. " Specific type: " .. SLOT_TYPE_GUILD_SPECIFIC_ITEM)
        --d(inventorySlot.sellerName)
        --d(TRADING_HOUSE:IsAtTradingHouse())
        if TRADING_HOUSE:IsAtTradingHouse() and inventorySlot.sellerName and type ~= SLOT_TYPE_GUILD_SPECIFIC_ITEM then
            -- Also prevent whisper for guild tabard's
            local function postponed()
                AddMenuItem(GetString(SI_SOCIAL_LIST_PANEL_WHISPER), function() whisperToSeller(inventorySlot) end, MENU_ADD_OPTION_LABEL)
                ShowMenu()
            end

            zo_callLater(postponed, 50) -- To avoid double call of ShowMenu()
        end
    end

    ZO_PreHook("ZO_InventorySlot_ShowContextMenu", onHookNeContextMenuItem)
end

--- Updates sliders configuration in order to show their values.
local function addValueLabel(widget)
    local valueLabel = CreateControlFromVirtual(widget:GetName() .. "TweakItValueLabel", widget:GetParent(), "TweakItValueLabel")
    valueLabel:SetAnchor(LEFT, widget, RIGHT, 10, 0)
    valueLabel:SetText("Empty")
    local function update()
        valueLabel:SetText(widget:GetValue())
    end

    widget:SetHandler("OnUpdate", update)
    --zo_callLater(function() d(valueLabel:GetName()) end, 2000)
end

local function addValueLabels()
    addValueLabel(GetControl("Options_Video_Gamma_AdjustmentSlider"))
    addValueLabel(GetControl("Options_Video_CustomScaleSlider"))
end

-----------------------------------------------------------------------------------------
-- OnAddOnLoaded  --
-----------------------------------------------------------------------------------------
local function OnAddOnLoaded(_event, loadedAddonName)
    if loadedAddonName == ADDON_NAME then
        Initialize()
        configureTradeHouseContextHook()
        addValueLabels()
    end
    if loadedAddonName == "FilterIt" then
        TweakIt.addonsLoaded["FilterIt"] = true
    end
    if loadedAddonName == "pChat" then
        if oldD == CHAT_SYSTEM.Zo_AddMessage then
            hookChatForLinkEntending()
        end
    end
end

---------------------------------------------------------------------
-- Register Events --
---------------------------------------------------------------------
EVENT_MANAGER:RegisterForEvent(TweakIt.name, EVENT_ADD_ON_LOADED, OnAddOnLoaded)








